{% extends "stagesetting/base.html" %}
{% load i18n %}
{% block content %}
    <form action="{% url 'stagesetting_update' original.pk %}" method="post" accept-charset="utf-8">
        {% csrf_token %}
        {{ form.as_ul }}
        <input type="submit" value="{% trans 'Save' %}">
    </form>
{% endblock %}
